@@ -4,8 +4,8 @@ import discord
 
 from . import configs, _http, models, utils, exceptions
 
-from oauthlib.common import add_params_to_uri
 from quart import request, session, redirect, current_app
+from oauthlib.common import add_params_to_uri, generate_token
 
 
 class DiscordOAuth2Session(_http.DiscordOAuth2HttpClient):
@@ -55,7 +55,7 @@ class DiscordOAuth2Session(_http.DiscordOAuth2HttpClient):
 
     @staticmethod
     def __get_state():
-        return session.pop("DISCORD_OAUTH2_STATE", str())
+        return session.get("DISCORD_OAUTH2_STATE", str())
 
     async def create_session(
             self, scope: list = None, *, data: dict = None, prompt: bool = True,
@@ -93,7 +93,10 @@ class DiscordOAuth2Session(_http.DiscordOAuth2HttpClient):
         if not prompt and set(scope) & set(configs.DISCORD_PASSTHROUGH_SCOPES):
             raise ValueError("You should use explicit OAuth grant for passthrough scopes like bot.")
 
-        state = jwt.encode(data or dict(), current_app.config["SECRET_KEY"]).decode(encoding="utf-8")
+        data = data or dict()
+        data["__state_secret_"] = generate_token()
+
+        state = jwt.encode(data, current_app.config["SECRET_KEY"]).decode(encoding="utf-8")
 
         async with await self._make_session(scope=scope, state=state) as discord_:
             authorization_url, state = discord_.authorization_url(configs.DISCORD_AUTHORIZATION_BASE_URL)
@@ -210,21 +213,27 @@ class DiscordOAuth2Session(_http.DiscordOAuth2HttpClient):
         return await models.UserConnection.fetch_from_api()
 
     @staticmethod
-    async def fetch_guilds() -> list:
+    async def fetch_guilds(use_cache=True) -> list:
         """This method returns list of guild objects from internal cache if it exists otherwise makes an API
         call to do so.
 
+        Parameters
+        ----------
+        use_cache : bool, optional
+            can be set to False to avoid using the cache.  
+
         Returns
         -------
         list
             List of :py:class:`quart_discord.models.Guild` objects.
 
         """
-        user = models.User.get_from_cache()
-        try:
-            if user.guilds is not None:
-                return user.guilds
-        except AttributeError:
-            pass
+        if use_cache:
+            user = models.User.get_from_cache()
+            try:
+                if user.guilds is not None:
+                    return user.guilds
+            except AttributeError:
+                pass
 
         return await models.Guild.fetch_from_api()
